package com.airwatch.cordova.sdkplugin;

import android.util.Log;

import com.airwatch.sdk.AirWatchSDKException;

import java.util.Collection;

import org.apache.cordova.*;
import org.apache.cordova.PluginResult.Status;
import org.json.*;

public class AirwatchSDKPlugin extends CordovaPlugin {

    public static final String LOG_TAG = AirwatchSDKPlugin.class.getSimpleName();

    private CallbackContext persistedContext;
    private AirwatchBridge bridge;

    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        bridge = AirwatchBridge.getInstance();
        try {
            bridge.initAirwatchSDKManager(cordova.getActivity());
        } catch (AirWatchSDKException awsdke) {
            Log.e(LOG_TAG, "In initialize(), AirWatch SDK plugin failed to initialize", awsdke);
        }
    }

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {

        Log.w(LOG_TAG, "Entering execute(), action = " + action);

        if ("setPersistentCallback".equals(action)) {
            persistedContext = callbackContext;
            return true;

        // Connection information

        } else if ("groupId".equals(action)) {
            try {
                sendResult(bridge.getGroupId(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        } else if ("username".equals(action)) {
            try {
                sendResult(bridge.getUserName(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        } else if ("serverName".equals(action)) {
            try {
                sendResult(bridge.getServerName(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        // Restrictions

        } else if ("allowCopyPaste".equals(action)) {
            try {
                sendResult(bridge.allowCopyPaste(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        } else if ("allowOffline".equals(action)) {
            try {
                sendResult(bridge.isOfflineAllowed(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        } else if ("restrictDocumentToApps".equals(action)) {
            try {
                sendResult(bridge.restrictDocumentToApps(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        } else if ("allowedApplications".equals(action)) {
            try {
                sendResult(bridge.allowedApplications(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        // Custom settings

        } else if ("customSettings".equals(action)) {
            try {
                sendResult(bridge.getCustomSettings(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        // Uncategorized

        } else if ("isCompliant".equals(action)) {
            try {
                sendResult(bridge.isCompliant(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        } else if ("isCompromised".equals(action)) {
            try {
                sendResult(bridge.isCompromised(), callbackContext);
            } catch (AirWatchSDKException awsdke) {
                sendError(callbackContext);
            }
            return true;

        }
        
        return false;  // Returning false results in a "MethodNotFound" error.
    }

    private void sendResult(String resultText, CallbackContext callbackContext) {
        try {
            callbackContext.sendPluginResult(new PluginResult(Status.OK, resultText));
        } catch (Exception e) {
            Log.e(LOG_TAG, "Plugin exception", e);
            sendError(callbackContext);
        }
    }

    private void sendResult(Collection coll, CallbackContext callbackContext) {
        try {
            callbackContext.sendPluginResult(new PluginResult(Status.OK, new JSONArray(coll)));
        } catch (Exception e) {
            Log.e(LOG_TAG, "Plugin exception", e);
            sendError(callbackContext);
        }
    }

    private void sendResult(boolean result, CallbackContext callbackContext) {
        try {
            callbackContext.sendPluginResult(new PluginResult(Status.OK, result));
        } catch (Exception e) {
            Log.e(LOG_TAG, "Plugin exception", e);
            sendError(callbackContext);
        }
    }

    private void sendResult(int result, CallbackContext callbackContext) {
        try {
            callbackContext.sendPluginResult(new PluginResult(Status.OK, result));
        } catch (Exception e) {
            Log.e(LOG_TAG, "Plugin exception", e);
            sendError(callbackContext);
        }
    }

    private void sendError(CallbackContext callbackContext) {
        callbackContext.sendPluginResult(new PluginResult(Status.ERROR));
    }

    public void sendEvent(String eventName, JSONObject eventData) {
        PluginResult result = null;
        try {
            JSONObject event = new JSONObject();
            event.put("eventName", eventName);
            if (eventData != null) {
                event.put("eventData", eventData);
            }
            result = new PluginResult(Status.OK, event);
        } catch (JSONException je) {
            result = new PluginResult(Status.ERROR, "JSONException marshalling event data: " + je.getMessage());
        } finally {
            result.setKeepCallback(true);
            persistedContext.sendPluginResult(result);
        }
    }
}
